# init.py -- Initialize the replicator
#
# Richard Brooksby, Ravenbrook Limited, 2000-12-08.
#
# $Id: //info.ravenbrook.com/project/p4dti/version/1.0/code/replicator/init.py#5 $
#
# This file is copyright (c) 2001 Perforce Software, Inc.  All rights  
# reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1.  Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
# 
# 2.  Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in
#     the documentation and/or other materials provided with the
#     distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# HOLDERS AND CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
# OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
# TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH
# DAMAGE.
#

import p4
import re
import replicator

from config import *

if dt_name == "TeamTrack":

    import configure_teamtrack
    import dt_teamtrack

    teamtrack_config, p4_config, replicator_config, jobspec = \
        configure_teamtrack.configuration \
        (
         administrator_address = administrator_address,
         changelist_url = changelist_url,
         closed_state = closed_state,
         log_file = log_file,
         p4_client_executable = p4_client_executable,
         p4_password = p4_password,
         p4_port = p4_port,
         p4_server_description = p4_server_description,
         p4_user = p4_user,
         poll_period = poll_period,
         replicate_p = replicate_p,
         replicator_address = replicator_address,
         replicated_fields = replicated_fields,
         smtp_server = smtp_server,
         start_date = start_date,
         teamtrack_password = teamtrack_password,
         teamtrack_server = teamtrack_server,
         teamtrack_user = teamtrack_user,
         verbose = verbose,
         )
    
    dt = dt_teamtrack.dt_teamtrack(rid, sid, teamtrack_config)

elif dt_name == "Bugzilla":

    import configure_bugzilla
    import dt_bugzilla

    bugzilla_config, p4_config, replicator_config, jobspec = \
        configure_bugzilla.configuration \
        (
         administrator_address = administrator_address,
         changelist_url = changelist_url,
         closed_state = closed_state,
         log_file = log_file,
         p4_client_executable = p4_client_executable,
         p4_password = p4_password,
         p4_port = p4_port,
         p4_server_description = p4_server_description,
         p4_user = p4_user,
         poll_period = poll_period,
         replicate_p = replicate_p,
         replicator_address = replicator_address,
         replicated_fields = replicated_fields,
         rid = rid,
         sid = sid,
         smtp_server = smtp_server,
         start_date = start_date,
         dbms_host = dbms_host,
         dbms_database = dbms_database,
         dbms_port = dbms_port,
         dbms_user = dbms_user,
         dbms_password = dbms_password,
         bugzilla_directory = bugzilla_directory,
         verbose = verbose,
        )
    
    dt = dt_bugzilla.dt_bugzilla(bugzilla_config)

else:

    raise "Configuration error", "The defect tracker '%s' is not supported." % dt_name

p4_interface = p4.p4(p4_config)

r = replicator.replicator(rid, dt, p4_interface, replicator_config)

# Take a look at the old jobspec.  If it has no P4DTI-rid field, then we assume
# that this is the first time the P4DTI has been run.  If so, check for the
# existence of jobs; if there are any, don't go ahead with the change to the
# jobspec but instead warn the administrator.  See job000219.

field_re = re.compile('^Fields[0-9]+$')
p4dti_rid_re = re.compile('^[0-9]+ P4DTI-rid ')
first_time = 1
for k, v in r.p4.run('jobspec -o')[0].items():
    if field_re.match(k) and p4dti_rid_re.match(v):
        first_time = 0
        break
if first_time and r.p4.run('jobs'):
    raise "Configuration error", ("You must delete your Perforce jobs before "
                                  "running the P4DTI for the first time.  See "
                                  "section 3.2.2 of the Administrator's "
                                  "Guide.")

r.p4.run('jobspec -i', [jobspec])

r.init()

#
# B. Document History
# 
# 2000-11-30  GDR  Added changelist_url configuration parameter.
# 
# 2000-12-04 GDR Alphabetized parameters.  Added replicator_address.  Made sure
# to pass all parameters to configure_teamtrack.
#
# 2000-12-08 RB  Moved configuration parameters to config.py.  Merged separate
# TeamTrack and Bugzilla activation scripts.
#
# 2001-01-11 NB  Bugzilla startup has changed because configure_bugzilla
# now opens the MySQL connection.
#
# 2001-01-18 NB  bugzilla_user has gone.
#
# 2001-01-25 NB  Added bugzilla_directory.
#
# 2001-02-04 GDR Added start_date parameter.
#
# 2001-02-16 NB Added replicate_p configuration parameter.
#
# 2001-03-02  RB    Transferred copyright to Perforce under their license.
